module AppHill

   class Config
   		@default_copyright_name  = 'AppHill'
   		@default_copyright_link  = 'http://apphill.io'
   		
   		def self.copyright_year
	   	   Time.now.year
   		end
   		
   		def self.copyright_name
   		   @default_copyright_name
   		end
   		
   		def self.copyright_link
   		   @default_copyright_link
   		end
   end

   class IOS
   
      def self.new_project (appname, config)
         Dir.mkdir appname
         Dir.mkdir "#{appname}/App"
         Dir.mkdir "#{appname}/AppTests"

         Dir.mkdir "#{appname}/AppTests/en.lproj"
         Dir.mkdir "#{appname}/App/en.lproj"
         Dir.mkdir "#{appname}/App/Images.xcassets"
         Dir.mkdir "#{appname}/App/Images.xcassets/AppIcon.appiconset"
         Dir.mkdir "#{appname}/App/Images.xcassets/LaunchImage.launchimage"

         AppHill::IO.generate_file "#{appname}/App/Images.xcassets/AppIcon.appiconset/Contents.json", "App/App/app-icons.json", config
         AppHill::IO.generate_file "#{appname}/App/Images.xcassets/LaunchImage.launchimage/Contents.json", "App/App/launch-images.json", config
         AppHill::IO.generate_file "#{appname}/App/en.lproj/InfoPlist.strings", "App/App/InfoPlist.strings", config

         AppHill::IO.generate_file "#{appname}/App/AppDelegate.h", "App/App/AppDelegate.h", config
         AppHill::IO.generate_file "#{appname}/App/AppDelegate.m", "App/App/AppDelegate.m", config
         AppHill::IO.generate_file "#{appname}/App/main.m", "App/App/main.m", config
         AppHill::IO.generate_file "#{appname}/App/App-Info.plist", "App/App/App-Info.plist", config
         AppHill::IO.generate_file "#{appname}/App/App-Prefix.pch", "App/App/App-Prefix.pch", config

         AppHill::IO.generate_file "#{appname}/AppTests/AppTests-Info.plist", "App/AppTests/AppTests-Info.plist", config
         AppHill::IO.generate_file "#{appname}/AppTests/AppTests.m", "App/AppTests/AppTests.m", config
         AppHill::IO.generate_file "#{appname}/AppTests/en.lproj/InfoPlist.strings", "App/AppTests/InfoPlist.strings", config

		 proj = Xcodeproj::Project.open(File.expand_path('../templates/App/App.xcodeproj', __FILE__))
         proj.save "#{appname}/#{appname}.xcodeproj"         
      end  
      
   end

   class IO
	   @templates_dir 		= File.expand_path('../templates', __FILE__)
	   @license_filename 	= "LICENSE"
	   
       class BIND < OpenStruct
          def get_binding
             return binding()
          end
       end
	   	   
	   def self.init_app (appname)
          license =  self.license(:copyright_year => AppHill::Config.copyright_year, 
								  :copyright_name => AppHill::Config.copyright_name, 
								  :copyright_link => AppHill::Config.copyright_link)

	      AppHill::IOS.new_project appname, {:app_id => "io.apphill.#{appname}", 
	      									 :app_name => appname,
	      									 :license => license}
	   end

	   def self.appname_available(appname)
	      return !File.directory?(appname)
	   end
	   
	   def self.appname_is_valid(appname)
	      !appname.match(/\s/)
	   end
	   
	   def self.template_filepath (filename)
	      File.join @templates_dir, filename + ".erb"
	   end   

	   def self.template_file (filename)
	      filepath = self.template_filepath filename
	      file     = File.open(filepath, 'r').read
	      return file
	   end

	   def self.template_bind(filename, data)
	      template = self.template_file filename
	      ERB.new(template).result(self::BIND.new(data).get_binding)
       end
       
	   def self.license (data)
	      self.template_bind @license_filename, data
	   end
	   
	   def self.generate_file(destination, filename, data)
	       content = self.template_bind filename, data
           File.open(destination, 'w') {|f| f.write(content) }
	   end
   end
   
   class CLI < Thor

      desc "create APPNAME", "Create a new app"
      def create (appname)
         #abort ("App #{appname} already exists") unless AppHill::IO.appname_available appname
         #abort ("Your app name should not contain spaces") unless AppHill::IO.appname_is_valid appname
	     
        AppHill::IO.init_app appname
      end
      
   end
end